import { Component, OnInit, ViewChild } from '@angular/core';
import {
  IonHeader,
  IonToolbar,
  IonTitle,
  IonContent,
  IonGrid,
  IonRow,
  IonCol,
  IonCard,
  IonButtons,
  IonMenuButton,
  AlertController,
} from '@ionic/angular/standalone';
import {ColumnMode, DatatableComponent, NgxDatatableModule} from '@swimlane/ngx-datatable';
import { PengukuranService } from '../services/pengukuran.service';
import { Storage } from '@ionic/storage-angular';
import { CustomDateModule } from './custom-date.module';

@Component({
  selector: 'app-tabel',
  templateUrl: './tabel.page.html',
  styleUrls: ['./tabel.page.scss'],
  standalone: true,
  imports: [
    IonHeader,
    IonToolbar,
    IonTitle,
    IonContent,
    IonGrid,
    IonRow,
    IonCol,
    IonCard,
    NgxDatatableModule,
    IonButtons,
    IonMenuButton,
    CustomDateModule
  ],
})
export class TabelPage implements OnInit {

  @ViewChild(DatatableComponent) table!: DatatableComponent;
  loadingIndicator = false;
  rows: any[] = [];
  columns: any[] = [];
  restructuredData: any[] = [];

  // Mapping of field names to human-friendly labels
  fieldMapping: { [key: string]: string } = {
    'berat_badan': 'Berat Badan',
    'tinggi_badan': 'Tinggi Badan',
    'imt': 'IMT',
    'lingkar_perut': 'Lingkar Perut',
    'tekanan_darah_diastole': 'Tekanan Darah Diastole',
    'tekanan_darah_sistole': 'Tekanan Darah Sistole',
    'frekuensi_nafas': 'Frekuensi Nafas',
    'nadi': 'Nadi',
    'kadar_gula_darah': 'Kadar Gula Darah',
    'waktu_pengambilan_gula_darah': 'Waktu Pengambilan Gula Darah',
    'kolesterol_total': 'Kolesterol Total',
    'kolesterol_ldl': 'Kolesterol LDL',
    'kolesterol_hdl': 'Kolesterol HDL',
    'trigliserida': 'Trigliserida',
    'kadar_hb': 'Kadar HB',
    'kadar_asam_urat': 'Kadar Asam Urat',
    'hba1c': 'HBA1C',
    'golongan_darah': 'Golongan Darah',
    'kondisi_umum': 'Kondisi Umum',
    'keluhan_perasaan': 'Keluhan Perasaan'
  };

  // Mapping for waktu_pengambilan_gula_darah field values
  waktuPengambilanMapping: { [key: string]: string } = {
    'puasa': 'Puasa minimal 8 jam',
    'makan': '1-2 jam setelah makan',
    'gds': 'Gula darah sewaktu/ GDS',
    'tidur': 'Sebelum berangkat tidur malam'
  };

  constructor(
    private pengukuranService: PengukuranService,
    private storage: Storage,
    private alertController: AlertController
  ) { }

  async ngOnInit() {
    await this.storage.create();
    this.getPengukuranData();
  }

  async getPengukuranData() {
    this.loadingIndicator = true;
    const idIdentitas = await this.storage.get('id_identitas');
    this.pengukuranService.getPengukuranByIdentitasId(idIdentitas).subscribe(
      (res: any) => {
        this.rows = [...res.data];
        this.restructureData(this.rows);
        this.loadingIndicator = false;
      },
      async (error: any) => {
        console.error('Error:', error);
        this.loadingIndicator = false;
        const alert = await this.alertController.create({
          header: 'Error',
          message: 'Failed to load data.',
          buttons: ['OK']
        });
        await alert.present();
      }
    );
  }

  restructureData(data: any[]) {
    const keys = Object.keys(data[0]).filter(key => key !== 'id' && key !== 'created_at' && key !== 'updated_at' && key !== 'deleted_at' && key !== 'identitas_id');
    const dates = data.map(item => item.created_at);
    this.columns = [{ prop: 'property', name: 'Komponen' }, ...dates.map(date => ({ prop: date, name: new Date(date).toLocaleString() }))];

    this.restructuredData = keys.map(key => {
      const row: any = { property: this.fieldMapping[key] || key };
      data.forEach(item => {
        if (key === 'waktu_pengambilan_gula_darah') {
          row[item.created_at] = this.waktuPengambilanMapping[item[key]] || '';
        } else {
          row[item.created_at] = item[key];
        }
      });
      return row;
    });
  }

  protected readonly ColumnMode = ColumnMode;
}
